function C2p29_PSET4_1b

%%
% ********************************************************************
% Course Homework
%               2.29: Numerical Fluid Mechanincs, Spring 2007
% Problem Set #4, Problem #1
% 05/04/07
% ********************************************************************
% Reza Karimi:
%                            Graduate Student
%                    Mechanical Engineering Department
%                  Massachusetts Institute of Technology
% Email:  rkarimi@mit.edu
% Office: NE47-319, 500 Technology Square, Cambridge, MA 02139
% Phone: (617) 324-7605
% ********************************************************************

%%
close, clc, clear all

a=1;b=10;
Ratio=(a/b)^2;

dP_dz=1;
Mu=1;
Beta=dP_dz/Mu;
V_Factor=-Beta*a^2;
Q_Factor=V_Factor*a*b;
tau_Factor=a*dP_dz;

%% Analytical Value of Flow Rate
n_max=200;
n=[0:n_max];
alpha=(n+0.5)*pi/a; format long
Q_star_anal=2*(-2/3*a^3*b+sum(4/a./alpha.^5.*tanh(alpha*b)))/Q_Factor

%% Numerical Solution of Velocity Field
fprintf('\n Mesh Density (n=a/h) \t Normalized Flow Rate \t Flow Rate Relative Error \t Maximum Shear Stress')
fprintf('\n----------------------------------------------------------------------------------------------------------')
Q_k=[];h_k=[];Q_relative_error=[];

L=0; % Counter for mesh size
for n=2.^[5];
    L=L+1;
    N=n^2;
    
    x=linspace(0,a,n+1);
    y=linspace(0,b,n+1);

    h=1/n;                   % Represent The equation in Zeta\Eta Domain

    A=-2*(1+Ratio)*eye(N,N); % Represent The equation in Zeta\Eta Domain

    for I=1:N
        [Index,Coeff]=Node_Equation(I);
        k=1;
        for j=Index
            A(I,j)=A(I,j)+Coeff(k);
            k=k+1;
        end
    end

    B=ones(N,1);
    u=reshape((A\B)*h^2*a^2*Beta,n,n);
    u(n+1,n+1)=0;

    [Q]=Flow_Rate(u)/Q_Factor;

    if L>1
        Q_relative_error=100*(Q_prev/Q-1);
    else
        Q_relative_error=NaN;
    end
    Q_prev=Q;
    if n>1
        tau_max=Mu/(2*h)*(3*u(end,1)-4*u(end-1,1)+u(end-2,1));   % Note that u(end  ,1)=0
    else
        tau_max=Mu/(2*h)*(3*u(end,1)-4*u(end-1,1));   % Further note that    u(end-2,1)=0
    end

    fprintf('\n\t%d \t\t\t %+5.5f \t\t %+5.2e%% \t\t\t %+5.5f ',n,Q,Q_relative_error,tau_max/tau_Factor)
    Q_k(end+1)=[Q];h_k(end+1)=[h];

end
fprintf('\n\n')

%% Flow Rate Convergence Plot
loglog(h_k,abs(Q_k-Q_star_anal),'-*')
Fig_Set_Basic('Flow Rate Convergence vs Mesh Size','$h=\frac{a}{n}$','$\|Q^*-Q^*_{Analytical}\quad\|$')


%% Numerical Solution, Velocity Plot
V_max_numeric=min(min(u))/V_Factor
figure, contourf(x/a,y/a,u'/V_Factor,10);set(gca,'CLIM',[0 V_max_numeric]);
Fig_Set(['Normalized Velocity Contours ($V_z^*$) for $\frac{a}{h}=n=$' num2str(n)])

%% Numerical Solution, Shear Stress Plot

tau_xz=[
    zeros(1,n+1) 
    u(3:end,:)-u(1:end-2,:) 
    3*u(end,:)-4*u(end-1,:)+u(end-2,:)]/(2*a/n);

tau_yz=[zeros(n+1,1) u(:,3:end)-u(:,1:end-2) 3*u(:,end)-4*u(:,end-1)+u(:,end-2)]/(2*b/n);
c_max=max([ max(max(tau_xz)) max(max(tau_yz))])/tau_Factor;

figure
subplot(1,2,1)
contourf(x/a,y/a,tau_xz'/tau_Factor,10); set(gca,'CLIM',[0 c_max]);
Fig_Set([ '$\tau_{xz}^*$ for $\frac{a}{h}=n=$' num2str(n)]) %Normalized Shear Stress Contours 
subplot(1,2,2)
contourf(x/a,y/a,tau_yz'/tau_Factor,10); set(gca,'CLIM',[0 c_max]);
Fig_Set([ '$\tau_{yz}^*$ for $\frac{a}{h}=n=$' num2str(n)])

%% Analytical Solution, Velocity Plot, Shear Plot
n_max=200;
alpha=([0:n_max]+0.5)*pi/a;
%Ci=4/a.*(-1).^[0:n_max]./alpha.^3./cosh(alpha*b);
Ci    =4/a.*(-1).^[0:n_max]./alpha.^3./(1+exp(-2*alpha*b));
Ci_tau=4/a.*(-1).^[0:n_max]./alpha.^2./(1+exp(-2*alpha*b));

ua    =zeros(n+1);
tau_xz=zeros(n+1);
tau_yz=zeros(n+1);

i=0;
for xi=x
    i=i+1;j=0;
    for yi=y
        j=j+1;
        %ua(i,j)=1/2*((xi^2-a^2)+sum(Ci.*cosh(yi*alpha).*cos(xi*alpha)));
            ua(i,j)=1/2*((xi^2-a^2)+sum(Ci.*(exp(alpha*(yi-b)).*(1+exp(-2*alpha*yi))).*cos(xi*alpha)));
        tau_xz(i,j)=1/2*(2*xi-sum(Ci_tau.*(exp(alpha*(yi-b)).*(1+exp(-2*alpha*yi))).*sin(xi*alpha)));
        tau_yz(i,j)=1/2*(sum(Ci_tau.*(exp(alpha*(yi-b)).*(1-exp(-2*alpha*yi))).*cos(xi*alpha)));    
    end
end
V_max_analytical=min(min(ua))/V_Factor
figure, contourf(x/a,y/a,ua'/V_Factor,10); set(gca,'CLIM',[0 V_max_analytical]);
Fig_Set('Normalized Velocity Contours ($V_z^*$) from Analytical Solution')

c_max=max([ max(max(tau_xz)) max(max(tau_yz))]);

figure
subplot(1,2,1)
contourf(x/a,y/a,tau_xz'/tau_Factor,10); set(gca,'CLIM',[0 c_max]);
Fig_Set([ '$\tau_{xz}^*$: Analytical Value']) %Normalized Shear Stress Contours 
subplot(1,2,2)
contourf(x/a,y/a,tau_yz'/tau_Factor,10); set(gca,'CLIM',[0 c_max]);
Fig_Set([ '$\tau_{yz}^*$: Analytical Value'])

%% Functions
%%%%%%%%%%%%%%%%%%%%
    function [Q]=Flow_Rate(u)
        if rem(n,2)==1
            if n==1
                Q=(2/3)^2*u(1,1)*4*a*b;      % Simpson's Rule
            else
                Q=sum(sum(u))/(n+1)^2*4*a*b; % Trapezoidal's Rule
            end
        else
            u_y=(sum(u)+sum(u(3:2:n-1,:))+3*sum(u(2:2:n,:)))/(3*n);
            Q  =(sum(u_y)+sum(u_y(3:2:n-1))+3*sum(u_y(2:2:n)))/(3*n)*4*a*b; % Simpson's Rule
        end
    end
%%%%%%%%%%%%%%%%%%%%%%
    function [Index,Coeff]=Node_Equation(I)
        [i,j]=I_2ij(I);
        [I1,Ac1]=ij_2I(i+1,j  );
        [I2,Ac2]=ij_2I(i-1,j  );
        [I3,Ac3]=ij_2I(i  ,j-1);
        [I4,Ac4]=ij_2I(i  ,j+1);
        Index=[I1 I2 I3 I4];
        Coeff=[Ac1 Ac2 Ac3*Ratio Ac4*Ratio];
    end
%%%%%%%%%%%%%%%%%%%%%%
    function [i,j]=I_2ij(I)   % Gives unknow i,j index from I index
        i=rem(I,n);      %    x: Variation on i
        if i==0
            i=n;
        end
        j=(I-i)/n+1;     %    y: Variation on j
    end
%%%%%%%%%%%%%%%%%%%%%%
    function [I,Ac]=ij_2I(i,j) % I is the node\unknown number, Ac meas whether the node is active (it is not a zero boundary node)
        if i==0
            i=2;
        end
        if j==0
            j=2;
        end
        if i>n
            i=[];
        end
        if j>n
            j=[];
        end
        I=i+n*(j-1);           % If I=[] -> Ac=[] which means the node velocity is zero
        Ac=1;
        if isempty(I)
            Ac=[];
        end
    end
%%%%%%%%%%%%%%%%%%%%%
    function Fig_Set(Title)
        Fig_Set_Basic(Title,'$\frac{x}{a}$','$\frac{y}{a}$')
        axis equal, axis([0 1 0 b/a]),colorbar
    end
%%%%%%%%%%%%%%%%%%%%%
    function Fig_Set_Basic(Title,XTITLE,YTITLE)
        xlabel(XTITLE,'Interpreter','latex','fontsize',20,'FontAngle','italic','FontWeight','bold','FontName','Times New Roman')
        ylabel(YTITLE,'Interpreter','latex','fontsize',20,'FontAngle','italic','FontWeight','bold','FontName','Times New Roman')
        title (Title,'Interpreter','latex','fontsize',16,'FontAngle','italic','FontWeight','bold','FontName','Times New Roman')
        set(gcf,'Position',[50 50 [25 25]*25],'color','w'),set(gca,'fontsize',12)
        box on, grid on,
    end

end